% -*- MATLAB -*-
% Time-stamp: "2018-07-31 20:57:16 fujiwara"
%----------------------------------------------------------------------
%  Copyright (C) 2018, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function [y,varargout] = lu( a )

  [row,col] = size( a );

  exfloat_size = 1+exfloat.precision64();

  mat   = uint64( 1:row*col*exfloat_size );
  work  = uint64( 1:(row+2)*exfloat_size );
  pivot = uint64( 1:row );

  for m=1:row
    for n=1:col
      idx = ((m-1)*col+(n-1))*exfloat_size;
      mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
    end
  end   

  exflib_mex_lu(mat,row,col,pivot,exfloat.precision64(),work);

  %------------------------------------------------------------
  % return y = U+L-I (L*U=P*A), and pivot is lost
  %------------------------------------------------------------
  l = min(row,col);
  if (nargout == 0) || (nargout == 1)

    y(row,col) = exfloat();

    for m=1:row
      for n=1:col
        idx = ((m-1)*col+(n-1))*exfloat_size;
        y(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
    end   
     
    return

  end

  % index from C to MATLAB
  pivot = pivot + 1;

  %------------------------------------------------------------
  % return Y=P*L, varargout{1}=U, where L*U=P*A
  %------------------------------------------------------------
  l = min(row,col);
  y(row,l) = exfloat();
  varargout{1} = exfloat();
  %varargout{1}(row,col) = exfloat();  % does not work

  if (nargout == 2)

    for m=1:l
      for n=1:m-1
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        y(pivot(m),n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
      y(pivot(m),m) = exfloat( 1 );
    end

    for m=l+1:row
      for n=1:l
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        y(pivot(m),n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
    end

    for m=1:l
      for n=m:col
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        varargout{1}(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
    end

  end

  %------------------------------------------------------------
  % return y=L, varargout{1}=U, varargout{2}=P, where L*U=P*A
  %------------------------------------------------------------
  if (nargout >= 3)

    for m=1:l
      for n=1:m
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        y(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
      y(m,m) = exfloat( 1 );
    end

    for m=l+1:row
      for n=1:l
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        y(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
    end % for m

    for m=1:l
      for n=m:col
        idx = ( (m-1)*col+(n-1) )*exfloat_size;
        varargout{1}(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
      end
    end

    varargout{2} = exfloat();
    for m=1:row
      varargout{2}(m,pivot(m)) = exfloat(1);
    end

  end

end

%  roundinfo = 0;
%  exflib_float_round_near(c.num, roundinfo, exfloat.precision64());

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
