% -*- MATLAB -*-
% Time-stamp: "2015-11-08 20:31:33 user"
%----------------------------------------------------------------------
%  Copyright (C) 2015, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function x = mldivide ( a, b )
  
  % solve AX = B

  [arow,acol] = size ( a );
  [brow,bcol] = size ( b );
  
  if arow ~= brow
    error('Numbers of Rows Mismatched.');
    return
  end
  
  if arow < acol
    error('Warning: Solutions do NOT attain minimum norm in current implementation.');
  end

  exfloat_size = 1+exfloat.precision64;
  worknum = 2;
  work = uint64( 1:worknum*exfloat_size );
 
  if arow ~= acol
    
    array = uint64( 1:acol*(acol+bcol)*exfloat_size );
    for i=1:acol*(acol+bcol)*exfloat_size
        array(i) = 0;
    end
    
    % setup normal equation : A'*A X =A'*B
    for m=1:acol
        for n=1:acol
            for k=1:arow
                t = exfloat();
                exflib_float_mul(t.num, a(k,m).num, a(k,n).num, exfloat.precision64);
                idx = (m*(acol+bcol)+n)*exfloat_size; % a(m,n)
                exflib_float_add(array+idx, array+idx, t.num, exfloat.precision64); 
            end
        end
    end

    % A'*B
    for m=1:acol
        for n=1:bcol
            for k=1:brow
                t = exfloat();
                exflib_float_mul(t.num, a(k,m).num, b(k,n).num, exfloat.precision64);
                idx = (m*(acol+bcol)+acol+n)*exfloat_size; % a(m,acol+n)
                exflib_float_add(array+idx, array+idx, t.num, exfloat.precision64);
            end
        end
    end
    
    exflib_mex_gaussian_elimination(array, acol, bcol, exfloat.precision64, work);
  
  else
  
    array = uint64( 1:arow*(acol+bcol)*exfloat_size );
    for m=1:arow
        for n=1:acol
            idx = ((m-1)*(acol+bcol)+(n-1))*exfloat_size;
            array( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
        end
    end
  
    for m=1:brow
        for n=1:bcol
            idx = ((m-1)*(acol+bcol)+(acol+n-1))*exfloat_size;
            array( idx+1:idx+exfloat_size ) = b(m,n).num(1:exfloat_size);
        end
    end
  
    exflib_mex_gaussian_elimination(array, arow, bcol, exfloat.precision64, work);
    
  end % if arow ~= acol
  
  x(acol,bcol) = exfloat();
  for m=1:acol
      for n=1:bcol
          idx = ((m-1)*(acol+bcol)+(acol+n-1))*exfloat_size;
          x(m,n).num(1:exfloat_size) = array( idx+1:idx+exfloat_size );
      end
  end
  
  return
%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
