% -*- MATLAB -*-
% Time-stamp: "2015-11-08 08:34:39 user"
%----------------------------------------------------------------------
%  Copyright (C) 2015, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function c = mrdivide( a, b )

  % solve XA = B (X = c)

  [arow,acol] = size( a );
  [brow,bcol] = size( b );

  %------------------------------
  % Both a and b are scalars
  %------------------------------
  if( (arow == 1) & (acol == 1) & (brow == 1) & (bcol == 1) )

    a = exfloat( a );
    b = exfloat( b );
    c = exfloat();

    exflib_float_div(c.num, a.num, b.num, exfloat.precision64);

%  roundinfo = 0;
%  exflib_float_round_near(c.num, roundinfo, exfloat.precision64);

    return

  end

  %------------------------------
  % Only b is scalar
  %------------------------------
  if( (brow == 1) & (bcol == 1) )

    y = exfloat( b );

    one = exfloat( 1 );
    %one.num(1) = exfloat.BIAS;

    inv = exfloat();
    exflib_float_div(inv.num, one.num, y.num, exfloat.precision64);

    for m=1:arow
      for n=1:acol

        x = exfloat( a(m,n) );
        c(m,n) = exfloat();
        exflib_float_mul(c(m,n).num, x.num, inv.num, exfloat.precision64);
        
      end
    end

    return

  end

  %------------------------------
  % Both a and b are array
  %------------------------------
  if bcol ~= acol
    error('Numbers of Columns Mismatched.');
    return
  end
  
  if brow > bcol
    error('Warning: Solutions do NOT attain minimum norm in current implementation.');
  end

  %---------------------------------------------
  % build-up normal equation : B*B' X' = B*A'
  %---------------------------------------------
  % B*B'
  for m=1:brow
    for n=1:brow
      
      mat(m,n) = exfloat();
      for k=1:bcol
        mat(m,n) = mat(m,n) + b(m,k) * b(n,k);
      end

    end
  end

  % B*A'
  for m=1:brow
    for n=1:arow
      
      mat(m,brow+n) = exfloat();
      for k=1:bcol
        mat(m,brow+n) = mat(m,brow+n) + b(m,k) * a(n,k);
      end

    end
  end
  
  %------------------------------
  % Gaussian Elimination
  %------------------------------
  for k=1:brow-1
    
    % finding pivot
    max_idx = k;
    max_dig = abs( mat(k,k) );
    for l=k+1:brow
          
      if max_dig < abs( mat(l,k) )
        max_idx = l;
        max_dig = abs( mat(l,k) );
      end
    
    end
    
    % pivoting
    if k ~= max_idx
          
      for n=k:brow+arow
        tmp = mat(k,n);
        mat(k,n) = mat(max_idx,n);
        mat(max_idx,n) = tmp;
      end

    end
    
    for m=k+1:brow
      
      t = mat(m,k) / mat(k,k);
      
      for n=k+1:brow+arow
        mat(m,n) = mat(m,n) - t * mat(k,n);
        %mat(m,n) = mat(m,n) - mat(m,k) * mat(k,n) / mat(k,k);
      end

      mat(m,k) = 0;

    end

  end
  
  %------------------------------
  % Backward Substitution
  %------------------------------
  for k=brow:-1:1

    for n=1:arow
      
      for l=k+1:brow
        mat(k,brow+n) = mat(k,brow+n) - mat(k,l) * mat(l,brow+n);
      end
      
      mat(k,brow+n) = mat(k,brow+n) / mat(k,k);
      
    end

  end

  %------------------------------
  % Copy to Solution C
  %------------------------------
  for m=1:brow
    for n=1:arow
      c(n,m) = mat(m,brow+n);
    end
  end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
