% -*- MATLAB -*-
% Time-stamp: "2015-11-16 21:05:06 user"
%----------------------------------------------------------------------
%  Copyright (C) 2015, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function c = mtimes( a, b )

  exfloat_size = 1+exfloat.precision64;

  [arow,acol] = size( a );
  [brow,bcol] = size( b );

  if ( (arow == 1) & (acol == 1) & (brow == 1) & (bcol == 1) )
    x = exfloat( a );
    y = exfloat( b ); 
    c = exfloat();
    exflib_float_mul(c.num, x.num, y.num, exfloat.precision64);
    return;
  end

  %------------------------------
  % a is scalar
  %------------------------------
  if ( (arow == 1) & (acol == 1) )

    x = exfloat( a );

    N = brow*bcol;
    b_array = uint64( 1:N*exfloat_size );
    c_array = uint64( 1:N*exfloat_size );

    % pack b
    for n=1:bcol
      for m=1:brow
        idx = ( (n-1)*brow + (m-1) ) * exfloat_size;
        b_array( idx+1:idx+exfloat_size ) = b(m,n).num( 1:exfloat_size );
        % d = exfloat( b(m,n) );
        % b_array( idx+1:idx+exfloat_size ) = d.num( 1:exfloat_size );
      end
    end

    % execute
    exflib_mex_times_vs(c_array, b_array, x.num, N, exfloat.precision64);

    % unpack c
    for n=1:bcol
      for m=1:brow
        idx = ( (n-1)*brow + (m-1) ) * exfloat_size;
        c(m,n) = exfloat();
        c(m,n).num( 1:exfloat_size ) = c_array( idx+1:idx+exfloat_size );
      end
    end

    return

  end

  %------------------------------
  % b is scalar
  %------------------------------
  if ( (brow == 1) & (bcol == 1) )

    y = exfloat( b );

    N = arow*acol;
    a_array = uint64( 1:N*exfloat_size );
    c_array = uint64( 1:N*exfloat_size );

    % pack a
    for n=1:acol
      for m=1:arow
        idx = ( (n-1)*arow + (m-1) ) * exfloat_size;
        a_array( idx+1:idx+exfloat_size ) = a(m,n).num( 1:exfloat_size );
        %d = exfloat( a(m,n) );
        %a_array( idx+1:idx+exfloat_size ) = d.num( 1:exfloat_size );
      end
    end

    % execute
    exflib_mex_times_vs(c_array, a_array, y.num, N, exfloat.precision64);

    % unpack c
    for n=1:acol
      for m=1:arow
        idx = ( (n-1)*arow + (m-1) ) * exfloat_size;
        c(m,n) = exfloat();
        c(m,n).num( 1:exfloat_size ) = c_array( idx+1:idx+exfloat_size );
      end
    end

    return
  end

  %------------------------------
  % Both a and b are arrays
  %------------------------------
  if acol ~= brow
    error('Numbers of Columns Mismatched.');
    return
  end
 
  %------------------------------
  % Matrix Multiplication
  %------------------------------
  a_array = uint64( 1:arow*acol*exfloat_size );
  b_array = uint64( 1:brow*bcol*exfloat_size );

  crow = arow;
  ccol = bcol;
  c_array = uint64( 1:crow*ccol*exfloat_size );

  % pack a
  for m=1:arow
    for n=1:acol
        idx = ( (m-1)*acol + (n-1) ) * exfloat_size;
        a_array( idx+1:idx+exfloat_size ) = a(m,n).num( 1:exfloat_size );
        %d = exfloat( a(m,n) );
        %a_array( idx+1:idx+exfloat_size ) = d.num( 1:exfloat_size );
    end
  end

  % pack b
  for n=1:bcol
    for m=1:brow
        idx = ( (n-1)*brow + (m-1) ) * exfloat_size;
        b_array( idx+1:idx+exfloat_size ) = b(m,n).num( 1:exfloat_size );
        %d = exfloat( b(m,n) );
        %b_array( idx+1:idx+exfloat_size ) = d.num( 1:exfloat_size );
    end
  end

  % execute
  w = uint64( 1:exfloat_size );
  exflib_mex_mtimes(c_array, a_array, b_array, arow, acol, bcol, exfloat.precision64, w);

  % unpack c
  for n=1:ccol
    for m=1:crow
        idx = ( (n-1)*crow + (m-1) ) * exfloat_size;
        c(m,n) = exfloat();
        c(m,n).num( 1:exfloat_size ) = c_array( idx+1:idx+exfloat_size );
    end
  end

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
