% -*- MATLAB -*-
% Time-stamp: "2018-08-12 11:19:00 fujiwara"
%----------------------------------------------------------------------
%  Copyright (C) 2018, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function v = norm( a, varargin )

  [row,col] = size( a );

  if nargin == 1
    p = 2;
  elseif (nargin == 2) && ( strcmp(varargin{1},'inf') )
    p = 0;
  elseif (nargin == 2) && ( strcmp(varargin{1},'Inf') )
    p = 0;
  elseif (nargin == 2) && ( strcmp(varargin{1},'1') )
    p = 1;
  elseif (nargin == 2) && ( strcmp(varargin{1},'2') )
    p = 2;
  elseif (nargin == 2) && ( varargin{1} == 1 )
    p = 1;
  elseif (nargin == 2) && ( varargin{1} == 2 )
    p = 2;
  elseif (nargin == 2)
    error ('norm(): unrecognized vector norm.');
    return
  else
    error ('Invalid call to norm.');
    return
  end

  v = exfloat();

  %------------------------------
  % a is a row/column-vector
  %------------------------------
  if (row == 1) || (col == 1)

    length = max(row,col);

    if p == 1

      for n=1:length

        x = exfloat( a(n) );

        x.num(1) = bitand( x.num(1), exfloat.EXP_MASK );

        exflib_float_add(v.num, v.num, x.num, exfloat.precision64());

      end

      return

    end % 1-norm

    if p == 2

      x2  = exfloat();
      sum = exfloat();

      for n=1:length

        x = exfloat( a(n) );
        exflib_float_mul(x2.num, x.num, x.num, exfloat.precision64());
        exflib_float_add(sum.num, sum.num, x2.num, exfloat.precision64());

      end

      % temporary storage
      worksize = uint32( (exfloat.size64() + 1)*5 );
      work = zeros(worksize,1, 'uint64');

      exflib_float_sqrt(v.num, sum.num, exfloat.precision64(), work);

      return

    end % 2-norm

    if strcmp(p, 'Inf') || strcmp(p, 'inf')

      v = abs( a(1) );
      for n=2:length
        if v < abs(a(n))
          v = abs(a(n));
        end
      end

      return

    end % infty norm

    error ('unrecognized vector norm.');
    return

  end % (row == 1) || (col == 1)
  
  %------------------------------
  % a is an matrix
  %------------------------------
  %fprintf('%d-norm is Not yet implemented\n', p);

  %------------------------------------------------------------
  % 1-norm
  %------------------------------------------------------------
  if p == 1
    
    %v = sum( a(:,1) );
    v = exfloat(0);
    for i=1:row
      x = a(i,1);
      x.num(1) = bitand( x.num(1), exfloat.EXP_MASK );
      exflib_float_add(v.num, v.num, x.num, exfloat.precision64());
    end
    
    for j=2:col
      %s = sum( a(:,j) );
      s = exfloat(0);
      for i=1:row
        x = a(i,j);
        x.num(1) = bitand( x.num(1), exfloat.EXP_MASK );
        exflib_float_add(s.num, s.num, x.num, exfloat.precision64());
      end
      
      if v < s
        v = s;
      end
    end

    return

  end
  
  %------------------------------------------------------------
  % infty-norm
  %------------------------------------------------------------
  if p == 0

    %v = sum( a(1,:) );
    v = exfloat(0);
    for j=1:col
      x = a(1,j);
      x.num(1) = bitand( x.num(1), exfloat.EXP_MASK );
      exflib_float_add(v.num, v.num, x.num, exfloat.precision64());
   end
    
    for i=2:row
      %s = sum( a(i,:) );
      s = exfloat(0);
      for j=1:col
        x = a(i,j);
        x.num(1) = bitand( x.num(1), exfloat.EXP_MASK );
        exflib_float_add(s.num, s.num, x.num, exfloat.precision64());
      end
      
      if v < s
        v = s;
      end
    end

    return

  end
  
  if p ~= 2
    error ('unrecognized vector norm.');
    return
  end

  %------------------------------------------------------------
  % 2-norm
  %------------------------------------------------------------
  econ = 1;
  wantu = 0;
  wantv = 0;

  [row,col] = size( a );
  exfloat_size = 1+exfloat.precision64();

  if row >= col 

    minm = col;  % min(row+1,col);
    rho  = col;  % min(row,col);

    colex = col;
    uu    = uint64( 1 );
    vv    = uint64( 1 );

    mat = uint64( 1:row*colex*exfloat_size );
    s   = uint64( 1:colex*exfloat_size );

    for m=1:row
      for n=1:col
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
      end
    end   

    exflib_mex_svd(mat, row,colex, exfloat.precision64(), uu,s,vv, wantu,wantv);

  else % row < col ------------------------------------------------------------

    minm = row;  % min(col+1,row);
    rho  = row;  % min(col,row);

    rowex = row;
    uu = uint64( 1 );
    vv = uint64( 1 );

    mat = uint64( 1:col*rowex*exfloat_size );
    s   = uint64( 1:rowex*exfloat_size );

    for m=1:col
      for n=1:row
        idx = ((m-1)*rowex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(n,m).num(1:exfloat_size);
      end
    end   

    exflib_mex_svd(mat, col,rowex, exfloat.precision64(), uu,s,vv, wantu,wantv);

  end % row < col ------------------------------------------------------------

  v.num(1:exfloat_size) = s(1:exfloat_size);

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
