% -*- MATLAB -*-
% Time-stamp: "2018-01-03 15:21:11 user"
%----------------------------------------------------------------------
%  Copyright (C) 2015, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function str = num2str_scalar( x, output_prec, output_fmt )

    [ row, col ] = size( x );

    if row ~= 1 | col ~= 1
      error('error : num2str_scalar() : Argument should be scalar type');
    end

    if output_fmt == 'd'
      str = num2str( double ( x ) );
      return;
    end

    % Infty
    if x.num(1) == exfloat.EXP_MASK
      str = 'Infty';
      return;
    end

    % -Infty
    if x.num(1) == exfloat.NEGINFTY
      str = '-Infty';
      return;
    end

    % zero
    if (x.num(1) == 0) || (x.num(1) == exfloat.SIGN)

      str = [ '' ];
      if (x.num(1) == exfloat.SIGN)
        str = [ '-' ];
      end

      if (output_prec == 0)
  	str = [ str '0' ];
	return;
      end

      str = [ str '0.' ]; 

      for j=1:output_prec
	str = [ str '0' ];
      end
      return;
    end

    b64 = zeros(exfloat.size64+2,1, 'uint64');
%----------------------------------------
%    debug, 20151130 : After Matlab R2015b, argument passing manner for
%    a scalar variable is not call-by-reference (call-by-value?)
%----------------------------------------
%    deci_pow = int64(0);
%
%    % 1.(num) * 2^e --> i.(b64) * 10^d ('i' is 1 digit in radix 10)
%    exflib_convert_pow(x.num, deci_pow, b64, exfloat.precision64);
%----------------------------------------
    % The second element is not used (dummy).
    deci_pow_array = zeros(2, 'int64');

    % 1.(num) * 2^e --> i.(b64) * 10^d ('i' is 1 digit in radix 10)
    exflib_convert_pow(x.num, deci_pow_array, b64, exfloat.precision64);

    deci_pow = deci_pow_array(1);
%----------------------------------------

    flag = uint64(bitand(x.num(1), exfloat.SIGN)) ...
	+ uint64(bitshift(output_prec, 16)) ... % output_prec << 16
	+ uint64(exfloat.precision64+2);

    if output_fmt == 'e'

      flag = flag + exfloat.E_FORMAT;
      str  = exflib_scientific_format(flag, b64, deci_pow);

    elseif output_fmt == 'f'

      flag = flag + exfloat.F_FORMAT;
      str  = exflib_fixed_format(flag, b64, deci_pow);

    elseif (deci_pow < exfloat.default_fixed_scientific ) ...
           || (output_prec <= deci_pow) % scientific

      flag = flag + exfloat.E_FORMAT;
      str  = exflib_scientific_format(flag, b64, deci_pow);

    else % fixed

      flag = flag + exfloat.F_FORMAT;
      str  = exflib_fixed_format(flag, b64, deci_pow);

    end

end % function str=num2str_scalar( x, output_prec, output_fmt )

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
