% -*- MATLAB -*-
% Time-stamp: "2015-11-18 21:23:38 user"
%----------------------------------------------------------------------
%  Copyright (C) 2015, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function v = prod( a )

  [arow,acol] = size( a );
  exfloat_size = 1+exfloat.precision64;

  %------------------------------
  % a is a row-vector
  %------------------------------
  if(arow == 1)

    a_array = uint64( 1:acol*exfloat_size );

    % pack
    for i=1:acol
      idx = (i-1)*exfloat_size;
      a_array( idx+1:idx+exfloat_size ) = a(i).num( 1:exfloat_size );
    end

    v = exfloat( 0 );
    exflib_mex_prod_vector(v.num, a_array, acol, exfloat.precision64);

    return

  end

  %------------------------------
  % a is a column-vector
  %------------------------------
  if(acol == 1)

    a_array = uint64( 1:arow*exfloat_size );

    % pack
    for i=1:arow
      idx = (i-1)*exfloat_size;
      a_array( idx+1:idx+exfloat_size ) = a(i).num( 1:exfloat_size );
    end

    v = exfloat( 0 );
    exflib_mex_prod_vector(v.num, a_array, arow, exfloat.precision64);

    return

  end

  %------------------------------
  % a is a matrix
  %------------------------------
  a_array = uint64( 1:arow*acol*exfloat_size );
  v_array = uint64( 1:acol*exfloat_size );

  % pack
  for j=1:acol
    for i=1:arow
      idx = ( (j-1)*arow + (i-1) ) * exfloat_size;
      a_array( idx+1:idx+exfloat_size ) = a(i,j).num( 1:exfloat_size );
    end
  end

  % execute
  exflib_mex_prod_matrix(v_array, a_array, arow, acol, exfloat.precision64);

  % unpack
  for i=1:acol
    idx = (i-1)*exfloat_size;
    v(i) = exfloat();
    v(i).num( 1:exfloat_size ) = v_array( idx+1:idx+exfloat_size );
  end

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
