% -*- MATLAB -*-
% Time-stamp: "2018-08-16 19:55:11 fujiwara"
%----------------------------------------------------------------------
%  Copyright (C) 2018, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function [y,varargout] = qr( a, varargin )

  %------------------------------
  % economy size
  %------------------------------
  if (nargin == 1)
    econ = 0;
  elseif (nargin == 2) && ( strcmp(varargin{1},'econ') )
    econ = 1;
  elseif (nargin == 2) && (varargin{1} == 0)
    econ = 1;
  else
    error ('Invalid call to qr.');
    return
  end

  %------------------------------
  % output format
  %------------------------------
  if (nargout == 0) || (nargout == 1)
    wantq = 0;
    wantr = 0;
  elseif (nargout == 2)
    wantq = 1;
    wantr = 1;
  else
    error ('Invalid call to qr.');
    return
  end

  %------------------------------
  [row,col] = size( a );
  exfloat_size = 1+exfloat.precision64();

  %assert( row >= col );

  if row >= col

    if wantq == 0
      colex = col;
      q     = uint64( 1 );
      r     = uint64( 1 );
    elseif econ == 1
      colex = col;
      q     = uint64( 1:(row*col)*exfloat_size );
      r     = uint64( 1:(col*col)*exfloat_size );
    else 
      colex = row;
      q     = uint64( 1:(row*row)*exfloat_size );
      r     = uint64( 1:(row*row)*exfloat_size );
    end

    mat = uint64( 1:row*colex*exfloat_size );

    for m=1:row
      for n=1:col
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
      end
    end   
    if (wantq == 1) && (econ == 0)
      for n=(col+1):colex
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = 0;
      end
    end

    exflib_mex_qr(mat, row,colex, exfloat.precision64(), q,r, wantq,wantr);
    
    %------------------------------------------------------------
    % if(nargout == 0 || nargout == 1), then return y = R, where A = Q*R
    %------------------------------------------------------------
    if wantq == 0

      y(row,col) = exfloat();
      
      for m=1:row
        for n=1:col
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
        end
      end   

    elseif econ == 1

      %------------------------------------------------------------
      % nargout == 2, then return y=Q, varargout{1}=R, where A = Q*R
      % economy mode
      %------------------------------------------------------------

      y(row,colex) = exfloat();

      for m=1:row
        for n=1:col
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = q( idx+1:idx+exfloat_size );
        end
      end

      varargout{1} = exfloat();

      for m=1:col
        for n=1:col
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          varargout{1}(m,n).num(1:exfloat_size) = r( idx+1:idx+exfloat_size );
        end
      end

    else % wantu == 1 && econ == 0

      %------------------------------------------------------------
      % nargout == 2, then return y=Q, varargout{1}=R, where A = Q*R
      % full mode
      %------------------------------------------------------------

      y(row,row) = exfloat();

      for m=1:row
        for n=1:row
          idx = ( (m-1)*row+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = q( idx+1:idx+exfloat_size );
        end
      end

      varargout{1} = exfloat();

      for m=1:col
        for n=1:col
          idx = ( (m-1)*row+(n-1) )*exfloat_size;
          varargout{1}(m,n).num(1:exfloat_size) = r( idx+1:idx+exfloat_size );
        end
      end
      for m=(col+1):row
        for n=1:col
          idx = ( (m-1)*row+(n-1) )*exfloat_size;
          varargout{1}(m,n).num(1:exfloat_size) = 0;
        end
      end

    end % wantq == 1
    
  else % row < col

    if wantq == 0
      q     = uint64( 1 );
      r     = uint64( 1 );
    else
      q     = uint64( 1:(row*col)*exfloat_size );
      r     = uint64( 1:(col*col)*exfloat_size );
    end

    mat = uint64( 1:row*col*exfloat_size );

    for m=1:row
      for n=1:col
        idx = ((m-1)*col+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
      end
    end   

    exflib_mex_qr(mat, row,col, exfloat.precision64(), q,r, wantq,wantr);
    
    %------------------------------------------------------------
    % if(nargout == 0 || nargout == 1), then return y = R, where A = Q*R
    %------------------------------------------------------------
    if wantq == 0

      y(row,col) = exfloat();
      
      for m=1:row
        for n=1:col
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = mat( idx+1:idx+exfloat_size );
        end
      end   

    else % wantq == 1

      %------------------------------------------------------------
      % nargout == 2, then return y=Q, varargout{1}=R, where A = Q*R
      % economy / full mode (these are equivalent)
      %------------------------------------------------------------

      y(row,row) = exfloat();

      for m=1:row
        for n=1:row
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = q( idx+1:idx+exfloat_size );
        end
      end

      varargout{1} = exfloat();

      for m=1:row
        for n=1:col
          idx = ( (m-1)*col+(n-1) )*exfloat_size;
          varargout{1}(m,n).num(1:exfloat_size) = r( idx+1:idx+exfloat_size );
        end
      end

    end % wantq == 1

  end % row < col

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
