% -*- MATLAB -*-
% Time-stamp: "2018-08-12 10:45:11 fujiwara"
%----------------------------------------------------------------------
%  Copyright (C) 2018, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function r = rank( a )

  econ = 1;
  wantu = 0;
  wantv = 0;

  [row,col] = size( a );
  exfloat_size = 1+exfloat.precision64();

  if row >= col 

    minm = col;  % min(row+1,col);
    rho  = col;  % min(row,col);

    colex = col;
    u     = uint64( 1 );
    v     = uint64( 1 );

    mat = uint64( 1:row*colex*exfloat_size );
    s   = uint64( 1:colex*exfloat_size );

    for m=1:row
      for n=1:col
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
      end
    end   

    exflib_mex_svd(mat, row,colex, exfloat.precision64(), u,s,v, wantu,wantv);

  else % row < col ------------------------------------------------------------

    minm = row;  % min(col+1,row);
    rho  = row;  % min(col,row);

    rowex = row;
    u = uint64( 1 );
    v = uint64( 1 );

    mat = uint64( 1:col*rowex*exfloat_size );
    s   = uint64( 1:rowex*exfloat_size );

    for m=1:col
      for n=1:row
        idx = ((m-1)*rowex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(n,m).num(1:exfloat_size);
      end
    end   

    exflib_mex_svd(mat, col,rowex, exfloat.precision64(), u,s,v, wantu,wantv);

  end % row < col ------------------------------------------------------------

  % count number of non-zero elements in ordered 's'
  pr64 = exfloat.precision64();
  r = 0;

  eps = uint64( 1:exfloat_size );
  eps(1) = exfloat.BIAS - uint64(64*pr64);
  eps(2:exfloat_size) = 0;
  
  sigma = uint64( 1:exfloat_size );
  sigma(1:exfloat_size) = s( 1:exfloat_size );
  sigma(1) = bitand( sigma(1), exfloat.EXP_MASK);  % sigma = abs( largest singular value )
    
  if exflib_float_lt( sigma, eps, pr64 )
    return
  end
  
  r = 1;
  exflib_float_mul(eps, eps, sigma, pr64); % eps = eps * largest singular value

  for n=2:min(row,col)
    idx = (n-1)*exfloat_size;
    sigma(1:exfloat_size) = s( idx+1:idx+exfloat_size );
    sigma(1) = bitand( sigma(1), exfloat.EXP_MASK);
      
    if exflib_float_lt(sigma, eps, pr64)
      break;
    end
    
    r = r + 1;

  end

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
