% -*- MATLAB -*-
% Time-stamp: "2018-08-12 09:58:58 fujiwara"
%----------------------------------------------------------------------
%  Copyright (C) 2018, FUJIWARA,Hiroshi
%    <fujiwara@acs.i.kyoto-u.ac.jp>
%----------------------------------------------------------------------

function [y,varargout] = svd( a, varargin )

  %------------------------------
  % economy size
  %------------------------------
  if (nargin == 1)
    econ = 0;
  elseif (nargin == 2) && ( strcmp(varargin{1},'econ') )
    econ = 1;
  elseif (nargin == 2) && (varargin{1} == 0)
    econ = 1;
  else
    error ('Invalid call to svd.');
    return
  end

  %------------------------------
  % output format
  %------------------------------
  if (nargout == 0) || (nargout == 1)
    wantu = 0;
    wantv = 0;
  elseif (nargout == 3)
    wantu = 1;
    wantv = 1;
  else
    error ('Invalid call to svd.');
    return
  end

  %------------------------------
  [row,col] = size( a );
  exfloat_size = 1+exfloat.precision64();

  if row >= col 

    minm = col;  % min(row+1,col);
    rho  = col;  % min(row,col);

    if wantu == 0
      colex = col;
      u     = uint64( 1 );
      v     = uint64( 1 );
    elseif econ == 1
      colex = col;
      u     = uint64( 1:(row*colex)*exfloat_size );
      v     = uint64( 1:(colex*colex)*exfloat_size );
    else 
      colex = row;
      u     = uint64( 1:(row*colex)*exfloat_size );
      v     = uint64( 1:(colex*colex)*exfloat_size );
    end

    mat = uint64( 1:row*colex*exfloat_size );
    s   = uint64( 1:colex*exfloat_size );

    for m=1:row
      for n=1:col
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(m,n).num(1:exfloat_size);
      end
      for n=(col+1):colex
        idx = ((m-1)*colex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = 0;
      end
    end   

    exflib_mex_svd(mat, row,colex, exfloat.precision64(), u,s,v, wantu,wantv);

    %------------------------------------------------------------
    % if(nargout == 0 || nargout == 1), then return y = s, where A = U*S*V'
    %------------------------------------------------------------
    if wantu == 0

      y(col,1) = exfloat();
    
      for m=1:col
        idx = (m-1)*exfloat_size;
        y(m,1).num(1:exfloat_size) = s( idx+1:idx+exfloat_size );
      end   

      return

    else % wantu

    %------------------------------------------------------------
    % nargout == 3, then return y=U, varargout{1}=S, varargout{2}=V, where A = U*S*V'
    %------------------------------------------------------------

      y(row,colex) = exfloat();

      for m=1:row
        for n=1:colex
          idx = ( (m-1)*colex+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = u( idx+1:idx+exfloat_size );
        end
      end

      varargout{1} = exfloat();

      for m=1:row
        for n=1:col
          if m == n
            idx = (m-1)*exfloat_size;
            varargout{1}(m,m).num(1:exfloat_size) = s( idx+1:idx+exfloat_size );
          else
            varargout{1}(m,n) = exfloat(0);
          end 
        end
      end

      varargout{2} = exfloat();

      for m=1:col
        for n=1:col
          idx = ( (m-1)*colex+(n-1) )*exfloat_size;
          varargout{2}(m,n).num(1:exfloat_size) = v( idx+1:idx+exfloat_size );
        end
      end

      return

    end % wantu == 1

  else % row < col ------------------------------------------------------------

    minm = row;  % min(col+1,row);
    rho  = row;  % min(col,row);

    if wantu == 0
      rowex = row;
      u = uint64( 1 );
      v = uint64( 1 );
    elseif econ == 1
      rowex = row;
      u = uint64( 1:(col*rowex)*exfloat_size );
      v = uint64( 1:(rowex*rowex)*exfloat_size );
    else 
      rowex = col;
      u     = uint64( 1:(col*rowex)*exfloat_size );
      v     = uint64( 1:(rowex*rowex)*exfloat_size );
    end

    mat = uint64( 1:col*rowex*exfloat_size );
    s   = uint64( 1:rowex*exfloat_size );

    for m=1:col
      for n=1:row
        idx = ((m-1)*rowex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = a(n,m).num(1:exfloat_size);
      end
      for n=(row+1):rowex
        idx = ((m-1)*rowex+(n-1))*exfloat_size;
        mat( idx+1:idx+exfloat_size ) = 0;
      end
    end   

    exflib_mex_svd(mat, col,rowex, exfloat.precision64(), u,s,v, wantu,wantv);

    %------------------------------------------------------------
    % return y = s, where A = U*S*V'
    %------------------------------------------------------------
    if wantu == 0

      y(row,1) = exfloat();
    
      for m=1:row
        idx = (m-1)*exfloat_size;
        y(m,1).num(1:exfloat_size) = s( idx+1:idx+exfloat_size );
      end   

      return

    else % wantu

    %------------------------------------------------------------
    % nargout == 3, then return y=U, varargout{1}=S, varargout{2}=V, where A = U*S*V'
    %------------------------------------------------------------

      y(row,row) = exfloat();

      for m=1:row
        for n=1:row
          idx = ( (m-1)*rowex+(n-1) )*exfloat_size;
          y(m,n).num(1:exfloat_size) = v( idx+1:idx+exfloat_size );
        end
      end

      varargout{1} = exfloat();

      for m=1:row
        for n=1:col
          if m == n
            idx = (m-1)*exfloat_size;
            varargout{1}(m,m).num(1:exfloat_size) = s( idx+1:idx+exfloat_size );
          else
            varargout{1}(m,n) = exfloat(0);
          end
        end
      end

      varargout{2} = exfloat();

      for m=1:col
        for n=1:rowex
          idx = ( (m-1)*rowex+(n-1) )*exfloat_size;
          varargout{2}(m,n).num(1:exfloat_size) = u( idx+1:idx+exfloat_size );
        end
      end

    end
    
  end % row < col ------------------------------------------------------------

end

%----------------------------------------------------------------------
% End of file
%----------------------------------------------------------------------
