/* -*- C -*- */
/* Time-stamp: "2015-11-08 16:52:35 user" */
/*----------------------------------------------------------------------*/
/*  Copyright (C) 2015, FUJIWARA Hiroshi                                */
/*    <fujiwara@acs.i.kyoto-u.ac.jp>                                    */
/*----------------------------------------------------------------------*/
#include "mex.h"
#include "exfloat.h"

#ifdef _OPENMP
#include <omp.h>
#endif

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  unsigned long long *const a = (unsigned long long*)mxGetData( prhs[0] );  
  const size_t N    = mxGetScalar( prhs[1] );  /* size */
  const size_t pr64 = mxGetScalar( prhs[2] );  /* precision64 */
  unsigned long long *const det = (unsigned long long*)mxGetData( prhs[3] );

  const size_t s64   = 1 + pr64;
  const int bytes    = s64 * sizeof(exflib_uQword);

  exflib_uQword *const w0 = (exflib_uQword*)malloc( bytes*2 );
  exflib_uQword *const w1 = w0 + s64;
  
  int i, j, k, l, p;

  int t = 0; /* exchange times of pivot  */

  /* a[i][j] corresponds to a[ (i*N+j)*s64 ] */

  /* forward elimination */
  for(k = 0; k < N-1; k++){
      
      /* pivoting */
      int max_idx = k;
      for(p = 0; p < s64; p++)
	  w0[p] = a[(k*N+k)*s64 + p];
      w0[0] &= EXFLIB_MASK_EXP;  /* abs( a[k][k] ) */

      /* finding pivot */
      for(l = k+1; l < N; l++){
	
	for(p = 0; p < s64; p++)
	  w1[p] = a[(l*N+k)*s64 + p];
	w1[0] &= EXFLIB_MASK_EXP; /* abs( a[l][k] ) */

	if( exflib_float_lt(w0, w1, pr64) ){
	    max_idx = l;
	    for(p = 0; p < s64; p++)
	      w0[p] = w1[p];
	}
      }

      if( w0[0] == 0 ){  /* singular matrix */
        for(p = 0; p < s64; p++)
          det[p] = 0;
        free( w0 );
        return;
      }

      /* exchanging pivot */
      t += 1;
      if( k != max_idx ){
	for(j = k; j < N; j++){
	  for(p = 0; p < s64; p++)
	    w0[p] = a[(k*N+j)*s64+p];
	  for(p = 0; p < s64; p++)
	    a[(k*N+j)*s64+p] = a[(max_idx*N+j)*s64+p];
	  for(p = 0; p < s64; p++)
	    a[(max_idx*N+j)*s64+p] = w0[p];
	}
      }

      for(i = k+1; i < N; i++){

	/* w0 = a[i][k] / a[k][k]; */
	exflib_float_div(w0, a+(i*N+k)*s64, a+(k*N+k)*s64, pr64);

        #pragma omp parallel for
        for(j = k+1; j < N; j++){
	  /* a[i][j] -= w0 * a[k][j]; */
	  exflib_float_mul(w1, w0, a+(k*N+j)*s64, pr64);
	  exflib_float_sub(a+(i*N+j)*s64, a+(i*N+j)*s64, w1, pr64);
	}

        /* a[i][k] = 0; */
	for(p = 0; p < s64; p++)
	  a[(i*N+k)*s64 + p] = 0;
      }
  }

  det[0] = EXFLIB_BIAS;
  for(p = 1; p < s64; p++)
    det[p] = 0;

  for(k = 0; k < N; k++)
    exflib_float_mul(det, det, a+(k*N+k)*s64, pr64);

  if( t%2 == 1 )
    det[0] ^= EXFLIB_MASK_SGN; /* det *= -1 */

  free( w0 );
  
  return;
}

/* End of file */
