/* -*- C -*- */
/* Time-stamp: "2018-08-15 08:40:19 fujiwara" */
/*----------------------------------------------------------------------*/
/*  Copyright (C) 2018, FUJIWARA Hiroshi                                */
/*    <fujiwara@acs.i.kyoto-u.ac.jp>                                    */
/*----------------------------------------------------------------------*/
#include "mex.h"
#include "exfloat.h"

/*----------------------------------------------------------------------*/
/* The following routine is copy from JAMA (A Java Matrix Package)      */
/* https://math.nist.gov/javanumerics/jama/                             */
/*----------------------------------------------------------------------*/


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  unsigned long long *const a   = (unsigned long long*)mxGetData( prhs[0] );  
  const size_t rows             = mxGetScalar( prhs[1] );
  const size_t cols             = mxGetScalar( prhs[2] );
  unsigned long long *const piv = (unsigned long long*)mxGetData( prhs[3] ); /* pivot */
  const size_t pr64             = mxGetScalar( prhs[4] );              /* precision64 */
  unsigned long long *const w   = (unsigned long long*)mxGetData( prhs[5] );  /* work */
  
  const size_t s64 = 1 + pr64;
  unsigned long long *const LUcolj = w;
  unsigned long long *const w1     = w  + rows*(1+pr64);
  unsigned long long *const w2     = w1 + (1+pr64);
  
  int i, j, k, q, p, kmax;

  for(i = 0; i < rows; i++)
      piv[i] = i;

  /* a[i][j] corresponds to a[ (i*cols+j)*s64 ] */

  for(j = 0; j < cols; j++){

      for(i = 0; i < rows; i++)
          for(p = 0; p < s64; p++)
	      LUcolj[i*s64+p] = a[(i*cols+j)*s64+p];

      for(i = 0; i < rows; i++){
	  kmax = (i < j)? i: j; /* min(i,j) */

	  for(p = 0; p < s64; p++)
	      w1[p] = 0;

	  for(k = 0; k < kmax; k++){
	      exflib_float_mul(w2, a+(i*cols+k)*s64, LUcolj+k*s64, pr64);
	      exflib_float_add(w1, w1, w2, pr64);
	  }

	  exflib_float_sub(LUcolj+i*s64, LUcolj+i*s64, w1, pr64);
	  for(p = 0; p < s64; p++)
	      a[(i*cols+j)*s64+p] = LUcolj[i*s64+p];
      }

      q = j;
      for(i = j+1; i < rows; i++){

	  for(p = 0; p < s64; p++)
	      w1[p] = LUcolj[i*s64 + p];
	  w1[0] &= 0x7fffffffffffffffULL;  /* abs( LUcolj[i] ) */

	  for(p = 0; p < s64; p++)
	      w2[p] = LUcolj[q*s64 + p];
	  w2[0] &= 0x7fffffffffffffffULL;  /* abs( LUcolj[q] ) */
	  
	  if( exflib_float_lt(w2, w1, pr64) )
	      q = i;
      }

      if(q != j){

	  for(k = 0; k < cols; k++){

	      for(p = 0; p < s64; p++)
		  w1[p] = a[(q*cols+k)*s64+p];
	      for(p = 0; p < s64; p++)
		  a[(q*cols+k)*s64+p] = a[(j*cols+k)*s64+p];
	      for(p = 0; p < s64; p++)
		  a[(j*cols+k)*s64+p] = w1[p];
	  }

	  k = piv[q];
	  piv[q] = piv[j];
	  piv[j] = k;
      }

      w2[0] = 0x3fffffffffffffffULL;
      for(p = 1; p < s64; p++)
	  w2[p] = 0;

      if( (a[(j*cols+j)*s64] != 0) || (a[(j*cols+j)*s64] != 0x8000000000000000ULL) ){

	  exflib_float_div(w1, w2, a+(j*cols+j)*s64, pr64);

	  for(i = j+1; i < rows; i++)
	      exflib_float_mul(a+(i*cols+j)*s64, a+(i*cols+j)*s64, w1, pr64);
      }
  }

  return;
}

/* End of file */
