/* -*- C -*- */
/* Time-stamp: "2015-11-08 20:29:37 user" */
/*----------------------------------------------------------------------*/
/*  Copyright (C) 2015, FUJIWARA Hiroshi                                */
/*    <fujiwara@acs.i.kyoto-u.ac.jp>                                    */
/*----------------------------------------------------------------------*/
#include "mex.h"
#include "exfloat.h"

/* exflib_mex_mtimes(c, a, b, crow, k, ccol, exfloat.precision64, w); */
/* a : row-wise, b,c : column-wise */

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  unsigned long long *const c = (unsigned long long*)mxGetData( prhs[0] );  
  unsigned long long *const a = (unsigned long long*)mxGetData( prhs[1] );  
  unsigned long long *const b = (unsigned long long*)mxGetData( prhs[2] );  
  const size_t N    = mxGetScalar( prhs[3] );  /* size */
  const size_t K    = mxGetScalar( prhs[4] );  /* size */
  const size_t M    = mxGetScalar( prhs[5] );  /* size */
  const size_t pr64 = mxGetScalar( prhs[6] );  /* precision64 */
  unsigned long long *const w = (unsigned long long*)mxGetData( prhs[7] );  

  /* a : row-wise,    N*K */
  /* b : column-wise, K*M */
  /* c : column-wise, N*M */
  
  int i, j, k, p;

  for(p = 0; p < N*M*(1+pr64); p++)
    c[p] = 0;

  for(i = 0; i < N; i++){
    for(j = 0; j < M; j++){
      for(k = 0; k < K; k++){

	/* c[i][j] += a[i][k]*b[k][j]; */

        const unsigned long long *const aa = a + (i*K+k)*(1+pr64);
        const unsigned long long *const bb = b + (j*K+k)*(1+pr64);
	unsigned long long *const cc       = c + (j*N+i)*(1+pr64);

        exflib_float_mul(w, aa, bb, pr64);
        exflib_float_add(cc, cc, w, pr64);
      }
    }
  }
  
  /* exflib_ulong roundinfo = exflib_float_add( c, a, b, N ); */
  /* exflib_float_round_near(c, roundinfo, N); */

  return;
}

/* End of file */
