/* -*- C -*- */
/* Time-stamp: "2018-09-02 10:17:07 fujiwara" */
/*----------------------------------------------------------------------*/
/*  Copyright (C) 2018, FUJIWARA Hiroshi                                */
/*    <fujiwara@acs.i.kyoto-u.ac.jp>                                    */
/*----------------------------------------------------------------------*/
#include "mex.h"
#include "exfloat.h"
#include <stdio.h>
#include <stdlib.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  unsigned long long *const a = (unsigned long long*)mxGetData( prhs[0] );  
  const size_t N    = mxGetScalar( prhs[1] );  /* length */
  const size_t pr64 = mxGetScalar( prhs[2] );  /* precision64 */

  int i, j, p;

  union eight_bytes_union {
      double d;
      exflib_uQword pd;
  } eight_bytes;

  const size_t rand_size = sizeof( RAND_MAX );
  const double reciprocal = ( (double)1 )/(double)RAND_MAX;

  for(i = 0; i < N; i++){

    size_t idx = i*(1+pr64);
    unsigned long long *const f = a+idx;

    /*********************/
    /* fractional part   */
    /*********************/
    if( rand_size == 4 ){

      for(p = 1; p <= pr64; p++){
        f[p] = rand();
        f[p] <<= 32;
        f[p] += rand();
      }

    } else if(rand_size == 8) {

      for(p = 1; p <= pr64; p++)
        f[p] = rand();

    } else {
        fprintf(stderr, "exflib_mex_rand(): not implemented size\n");
        exit(1);
    }

    /*********************/
    /* exponent part     */
    /*********************/
    eight_bytes.d = ( (double)rand() )*reciprocal;

    f[0] = ( (eight_bytes.pd >> 52)-0x3ff ) + EXFLIB_BIAS;
    f[1] = (eight_bytes.pd << 12) | (f[1] >> 52);
  }

  return;
}

/* End of file */
