/* -*- C++ -*- */
/* Time-stamp: "2018-08-15 08:05:54 fujiwara" */

#ifndef EXFLOAT_H
#define EXFLOAT_H

typedef const          long long exflib_cQword;
typedef const unsigned long long exflib_cuQword;
typedef       unsigned long long exflib_uQword;
typedef                long long exflib_Qword;
typedef       unsigned      long exflib_ulong;

/******************************************************************/
/* Constatnt Identifier                                           */
/******************************************************************/

enum {
    EXFLIB_CONST_MAX_PR 	= 1024,
    EXFLIB_FUNCARG_MAX_PR64 	= 160,
    EXFLIB_FUNCARG_EXP_LEAST 	= -32,
    EXFLIB_FUNCLOG_INVTBL_MAXN 	= 500
};

static exflib_cuQword EXFLIB_TEN_19	= 0x8ac7230489e80000ULL;
static exflib_cuQword EXFLIB_TEN_18	= 0x0de0b6b3a7640000ULL;

static exflib_cuQword EXFLIB_MASK_SGN	= 0x8000000000000000ULL;
static exflib_cuQword EXFLIB_MASK_EXP   = 0x7fffffffffffffffULL;
static exflib_cuQword EXFLIB_DBL_EXP	= 0x7ff0000000000000ULL;
static exflib_cuQword EXFLIB_DBL_FLT	= 0x000fffffffffffffULL;
static exflib_cQword  EXFLIB_BIAS	= 0x3fffffffffffffffLL;

/******************************************************************/
/* Prototypes                                                     */
/******************************************************************/


#ifdef __cplusplus
#include <cstddef>	// definition of size_t
extern "C" {
#else
#include <stddef.h>	/* size_t */
#endif

    /* fixed point arithmetic */
    exflib_ulong exflib_array_add(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_array_sub(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_array_mul(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_array_mul_h(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_array_mul_k(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_uQword exflib_array_imul(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b, size_t n);
    exflib_ulong exflib_array_idiv(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b, size_t n, long *top);

    /* floating point arithmetic */
    exflib_ulong exflib_float_add(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_float_sub(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_float_mul(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);
    exflib_ulong exflib_float_imul(exflib_uQword c[], exflib_cuQword a[], exflib_Qword b, size_t n);
    exflib_ulong exflib_float_idiv(exflib_uQword c[], exflib_cuQword a[], exflib_Qword b, size_t n);
    exflib_ulong exflib_float_div(exflib_uQword c[], exflib_cuQword a[], exflib_cuQword b[], size_t n);

    /* floating point rounding */
    void exflib_float_round_chop(exflib_uQword a[], exflib_ulong hint, size_t n);
    void exflib_float_round_near(exflib_uQword a[], exflib_ulong hint, size_t n);
    void exflib_float_round_plus(exflib_uQword a[], exflib_ulong hint, size_t n);
    void exflib_float_round_minus(exflib_uQword a[], exflib_ulong hint, size_t n);

    /* floating point operation */
    long exflib_float_lt(exflib_cuQword a[], exflib_cuQword b[], size_t n);
    void exflib_float_ltoef(exflib_uQword a[], long b, size_t n);

    /* output ------------------------------*/
    void exflib_convert_pow(exflib_cuQword u[], exflib_Qword *d_int, exflib_uQword target[], long pr64);
    void exflib_scientific_format(exflib_uQword flag, exflib_uQword u[],
				 exflib_Qword u_pow, char output_string[]);
    void exflib_fixed_format(exflib_uQword flag, exflib_uQword u[],
			    exflib_Qword u_pow, char output_string[]);
    /* assign ------------------------------*/
    void exflib_float_etoef(const char *s, exflib_uQword u[], long pr64);
    void exflib_float_ctoef(const char *s, exflib_uQword u[], long pr64);
    /* 
       following functions are defined, and used in etoef.

       void exflib_float_htoef(const char *s, exflib_uQword u[], long pr64);
       void exflib_float_atoef(const char *s, exflib_uQword u[], long pr64);
    */

    /* exfloat <-> double */
    void exflib_float_dtoef(const double d, exflib_uQword u[], long pr64);
    double exflib_float_eftod(exflib_cuQword u[]);

    /* built-in functions */
    exflib_ulong exflib_float_sqrt(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_sin(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_cos(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_exp(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_pow_long(exflib_uQword y[], exflib_cuQword x[], long i, long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_pow(exflib_uQword p[], exflib_cuQword x[], exflib_cuQword y[], long pr64, exflib_uQword w[], exflib_uQword *inv_table);
    exflib_ulong exflib_float_log(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[], exflib_uQword *inv_table);

    exflib_ulong exflib_float_asin(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    exflib_ulong exflib_float_atan(exflib_uQword y[], exflib_cuQword x[], long pr64, exflib_uQword w[], exflib_uQword *inv_table);

    void exflib_init_reciprocal_table(exflib_uQword tbl[], long size);
	
    void exflib_float_modf(exflib_cuQword x[], exflib_uQword ip[], exflib_uQword fp[], long pr64);
    void exflib_float_fmod(exflib_uQword r[], exflib_cuQword d[], exflib_cuQword m[], long pr64, exflib_uQword w[]);
    void exflib_float_ceil(exflib_uQword ip[], exflib_cuQword x[], long pr64, exflib_uQword w[]);
    void exflib_float_floor(exflib_uQword ip[], exflib_cuQword x[], long pr64, exflib_uQword w[]);

    /* for atan functions */
    long exflib_atan_arg_reduction(exflib_uQword x[], long pr64);
    void exflib_atan_recover(exflib_uQword x[], long k, long pr64);
    void exflib_atan_horner(exflib_uQword x[], exflib_uQword v[], exflib_uQword table[], long pr64);

    /* utility */
    /* void exflib_version(); */
    const char *exflib_version();
    /* void exflib_float_rand(exflib_uQword x[], long pr64); */

#ifdef __cplusplus
}; // end of extern "C"
#endif

#endif
