% -*- MATLAB -*-
clear classes
%------------------------------------------------------------------------------
% Specify where *.mexw64, *.mexmaci64, and *.mexa64 files are located.
% Followings are typical settings for Windows, MacOSX, and Linux respectively.
% Uncomment either of them, and change 'kyoto' to your real account.
%------------------------------------------------------------------------------
addpath( strcat( pwd, '/exflib' ) );

fprintf('exfloat precision=%f\n', exfloat.get_precision());
fprintf('exfloat req prec =%d\n', exfloat.get_req_precision());
fprintf('exfloat size=%d bytes\n', exfloat.get_exfloat_byte());

%----------------------------------------------------------------------


N=10

fprintf('size=%d\n', N);

tic
%----------------------------------------------------------------------
% setup problem (Hilbert Matrix)
%----------------------------------------------------------------------
for n=1:N
  for m=1:N

    mat(n,m) = exfloat(1)/(n+m-1);

  end

  % RHS
  mat(n,N+1) = exfloat(1);

end

%----------------------------------------------------------------------
% Forward Elimination
%----------------------------------------------------------------------
for k=1:N-1
    
  % finding pivot
  max_idx = k;
  max_dig = abs( mat(k,k) );
  for l=k+1:N
    
    if max_dig < abs( mat(l,k) )
      max_idx = l;
      max_dig = abs( mat(l,k) );
    end
    
  end
  
  % pivoting
  if k ~= max_idx
    
    for n=k:N+1
      tmp = mat(k,n);
      mat(k,n) = mat(max_idx,n);
      mat(max_idx,n) = tmp;
    end

  end
  
  for m=k+1:N
    
    t = mat(m,k) / mat(k,k);
    
    for n=k+1:N+1
      mat(m,n) = mat(m,n) - t * mat(k,n);
    end

    mat(m,k) = 0;

  end

end

%----------------------------------------------------------------------
% Backward Substitution
%----------------------------------------------------------------------
for k=N:-1:1

  for l=k+1:N
    mat(k,N+1) = mat(k,N+1) - mat(k,l) * mat(l,N+1);
  end
  
  mat(k,N+1) = mat(k,N+1) / mat(k,k);
  
end

toc

%----------------------------------------------------------------------
% Solution
%----------------------------------------------------------------------

for n=1:N
  mat(n,N+1)
end

% End of file
